#include "gp32lib.h"

#include "gpmem.h"
#include "gpstdio.h"

//
static F_HANDLE	fHandle;
static DWORD	dwFileLength, dwFileOffset;

static LPBYTE	pPcmOffset;
static DWORD	dwPcmOffset;

static void UpdatePcmNextChunk(void)
{
	static DWORD dwChunkOffset = 0;

	DWORD dwLength, dwDummy;

	if (dwPcmChunkOffset == dwChunkOffset)
		return;

	dwChunkOffset = dwPcmChunkOffset;

	///////////////////////////////////////////////////////////////////////////
	if (dwFileOffset > (dwFileLength - 1))
	{
		GpFileSeek(fHandle, FROM_BEGIN, 0, NULL);

		dwLength = CHUNK_LENGTH;
		dwFileOffset = 0;
	}
	else
	{
		dwLength = dwFileLength - dwFileOffset;
		if (dwLength > CHUNK_LENGTH)
			dwLength = CHUNK_LENGTH;

		dwFileOffset += dwLength;
	}

	//
	dwPcmOffset += dwLength;
	if (dwPcmOffset > (dwPcmChunkLength - 1))
	{
		DWORD dwPcmBlock;

		dwPcmOffset -= dwPcmChunkLength;
		dwPcmBlock   = dwLength - dwPcmOffset;

		if (dwPcmBlock)
			GpFileRead(fHandle, pPcmOffset, dwPcmBlock, &dwDummy);

		if (dwPcmOffset)
			GpFileRead(fHandle, pPcmChunkBuffer, dwPcmOffset, &dwDummy);

		pPcmOffset = pPcmChunkBuffer + dwPcmOffset;

		return;
	}

	GpFileRead(fHandle, pPcmOffset, dwLength, &dwDummy);
	pPcmOffset += dwLength;
}

BOOL PlayPcmIRQ(const char* lpszFileName)
{
	DWORD dwLength;

	ERR_CODE result;

	// Get file size
	result = GpFileGetSize(lpszFileName, &dwFileLength);
	if (result != SM_OK)
		return FALSE;

	// Open file
	result = GpFileOpen(lpszFileName, OPEN_R, &fHandle);
	if (result != SM_OK)
		return FALSE;

	if (!CreatePcmChunk())
		return FALSE;

	dwPcmOffset = dwPcmChunkLength - CHUNK_LENGTH;
	pPcmOffset  = pPcmChunkBuffer + dwPcmOffset;

	// Read PcmChunkBuffer
	GpFileRead(fHandle, pPcmChunkBuffer, dwPcmOffset, &dwLength);

	dwFileOffset = dwLength;

	// install dma2-irq
    DMA2_ISR_Install();

	PlayPcmNextChunk();

	GpTimerOptSet(0, 90, 0, UpdatePcmNextChunk);
	GpTimerSet(0);

	return TRUE;
}
void StopPcmIRQ(void)
{
	GpTimerKill(0);

	// uninstall dma2-irq
    DMA2_ISR_Uninstall();
	DestroyPcmChunk();

    GpFileClose(fHandle);
}
